if (typeof(mc)=="undefined") { mc = {}; }

String.prototype.replaceAll = function(target, replacement) {
    return this.split(target).join(replacement);
};

var requiredAPIVersion = "0.4.1";

mc.context = {
    deviceInfo : {},
    position : { lat : 0, lon : 0 },
    isAdmin : false,
}

function onAPIReady(deviceId, deviceType) {
    
    try {
        
        mc.loggingEnabled = true;
        mc.alertErrors = true;
        mc.searchQuery = null;
        
        mc.context.deviceInfo.id = deviceId;
        mc.context.deviceInfo.type = deviceType;
        
        var observer = {
            
            onKeyboardVisibilityChange : function(visible) {
                mc.UI.onKeyboardVisibilityChange(visible);
            },
            
            onSearchQueryEntered : function(query) {
                mc.searchQuery = query;
                if (!query || query == "") {
                    mc.UI.handleSearch(query);
                }
            
            },
            
            onRightButtonClick : function(title) {
                mc.UI.onRightButton(title);
            },
            
            onBackButtonClick : function() {
                mc.UI.onBackButton();
            },
            
            onCancelButtonClick : function() {
            },
            
            onSearchCancel : function() {
                testLog("on search cancel");
            },
            
            onSearchButton : function() {
                testLog("on search button");
                //alert('on serach button');
                $("#search_button_tip").remove();
                mc.UI.handleSearch(mc.searchQuery);
            },
            
            onPullToRefresh : function() {
                testLog("on pull to refresh");
                //location.reload(true);
                mc.UI.onPullToRefresh();
            },
            
            onPositionUpdate : function(lat, lon, alt, accuracy) {
                var _first = false;
                if (!mc.context.position) { mc.context.position = {}; _first = true; };
                mc.context.position.lat = lat;
                mc.context.position.lon = lon;
                
                testLog("onPositionUpdate: " + lat.toFixed(3) + ", " + lon.toFixed(3) + ", " + accuracy.toFixed(2));
                
                //if (_first) {
                //    mc.UI.onFirstPositionReceived(mc.context.position);
                //}
            },
            
            onURIReceived : function(type, id, operation) {
                // alert("onURIReceived: " + type + ", " + id + ", " + operation);
                if (type == "world") {
                    mc.UI.openPage("worldinfo", { world_id : id }, mc.UI.TYPE_PUSH);
                }
                else if (type == "placed_world") {
                    mc.UI.openPage("placedworldinfo", { placedworld_id : id }, mc.UI.TYPE_PUSH);
                }
                else {
                    alert("Unknown type: " + type);
                }
            },
            
            onTabSelected : function(index) {
                testLog("onTabSelected: " + index);
                mc.UI.onTabSelected(index);            //mctest.tabIndex = index;
            },
            
            onBase64Image : function(path, data) {
                mc.UI.onBase64Image(path, data);
            },
            
            onAppEvent : function(eventName, data) {
                if (eventName == 'world_placed') {
                    mc.UI.onWorldPlaced(data);
                }
                else if (eventName == 'placed_world_saved') {
                    mc.UI.onWorldSaved(data);
                }
                
                testLog("onAppEvent: " + eventName + "(" + data + ")");
            }
        }
        
        $("#debuginfo").html('<div>'+location.href+'</div><div>Admin Mode: ' + mc.context.isAdmin + '</div>');
        
        
        mc.api.setObserver(observer);
        mc.api.setPullToRefreshDone();
        
        // Make sure searchbar is hidden
        mc.api.setSearchbarVisible(false);
        
        // Prepare page for specific device
        mc.UI.prepareDeviceLayout();
        
        setTimeout(function() { mc.api.setSearchbarVisible(false); }, 200);
        
        if (typeof(onMainReady)=="function") {
            onMainReady();
        }
        
        if (requiredAPIVersion != mc.apiVersion) {
            var txt = "Pre-release info: A newer version of the Minecraft Reality app is required for this API. Some features may be broken.\n\nContact 13th Lab for a more recent version.";
            setTimeout(function() { alert(txt); }, 200);
        }
        
    }
    catch (ex) {
        // DO NOT OUTPUT ERROR
        mc.log("Error: " + ex);
    }
}

mc.data = {
    pageData : {},
    
    clearCache : function() {
        localStorage.clear();
        mc.api.clearCache();
        mc.data.pageData = {};
    },
    
    baseURL : 'http://localhost:8082/Data/', //'http://api.minecraftreality.com/', //'http://minecloud.13thlab.com/',
    
    storeData : function(key, value, exdays) {
        if (!exdays) { exdays = 1000; }
        var exdate = new Date();
        exdate.setDate(exdate.getDate() + exdays);
        var c_value = escape(value) + ((exdays==null) ? "" : "; expires="+exdate.toUTCString());
        document.cookie = key + "=" + c_value;
    },
    
    setPageData : function(id, key, value) {
        var k = id + "_" + key;
        mc.data.pageData[k] = value;
    },
    
    getPageData : function(id, key) {
        var k = id + "_" + key;
        return mc.data.pageData[k];
    },
    
    getData : function(key, _default) {
        var i,x,y,ARRcookies = document.cookie.split(";");
        for (i=0;i<ARRcookies.length;i++) {
            x = ARRcookies[i].substr(0,ARRcookies[i].indexOf("="));
            y = ARRcookies[i].substr(ARRcookies[i].indexOf("=")+1);
            x = x.replace(/^\s+|\s+$/g,"");
            if (x == key) {
                return unescape(y);
            }
        }
        
        if (typeof(_default)!="undefined") {
            return _default;
        }
        return null;
    },
    
    clearData : function() {
        var cookies = document.cookie.split(";");
        for (var i = 0; i < cookies.length; i++) {
            var cookie = cookies[i];
            var eqPos = cookie.indexOf("=");
            var name = eqPos > -1 ? cookie.substr(0, eqPos) : cookie;
            document.cookie = name + "=;expires=Thu, 01 Jan 1970 00:00:00 GMT";
        }
    },
    
    setPlacedWorldStarred : function(id, shouldStar) {
        var signature = hex_md5 ? ("&sig=" + hex_md5(id+mc.context.deviceInfo.id+mc.api.apiSignKey)) : "";
        var op = shouldStar ? "star" : "unstar";
        var url = '';
        url += mc.data.baseURL + "/placedworlds/"+id+"/" + op + "/";
        url += "?device_id=" + mc.context.deviceInfo.id + "&device_type=" + encodeURIComponent(mc.context.deviceInfo.type);
        url += signature;
        url += "&callback=?";
        
        $.ajaxSetup({ cache : false });
        
        var _request = $.getJSON(url, function(data) { });
        _request.error(function() { alert('error'); });
        _request.complete(function() { });
    },
    
    request : function($elem, urlPath, notifyFromCache, storeToCache, respondWithCacheOnly) {
        storeToCache = (typeof(storeToCache)=="undefined") ? true : storeToCache;
        
        $.ajaxSetup({ cache : false }); // For now

        var baseURL = '';
        var debugAlert = false;
        var urlParts = urlPath.split("/");
        if (urlParts[0] == '[remote]') {
            baseURL = this.baseURL;
            urlParts.splice(0,1);
        }
        else if (urlParts[0] == '[local]'){
            baseURL = 'mcr/';
            urlParts.splice(0,1);
        }
        
        
        var url = baseURL + urlParts.join("/");
        
        //var sep = url.indexOf("?")==-1 ? "?" : "&";
        //url += sep + "callback=?";
        
        //alert(url);
        mc.api.log("Calling URL " + url);
        
        
        if (debugAlert) {
            alert(url);
        }
        
        mc.data.setPageData($elem.attr("id"), "current_url", url);
        
        
        if (notifyFromCache) {
            var cachedResponse = mc.data.cache.getObject(url);
            if (cachedResponse) {
                mc.UI.onData($elem, cachedResponse, true, null);
                if (respondWithCacheOnly) {
                    return;
                }
            }
        }
        
        var apiRequest = new mc.APIRequest($elem, url, storeToCache);
        apiRequest.sendRequest();
        
    },

    /*
    sendRequest : function(url, storeToCache) {
        var _request = $.getJSON(url, function(data) { if (storeToCache) { if (data) { mc.data.cache.storeObject(url, data); } } mc.UI.onData($elem, data, false, url); });
        _request.error(function() { mc.UI.onData($elem, null, false, url); });
        _request.complete(function() { mc.api.showNetworkIndicator(false); });
    },
     */
    
    cache : {
        storeObject : function(key, obj) {
            localStorage.setItem(key, JSON.stringify(obj));
        },
        
        getObject : function(key) {
            var objStr = localStorage.getItem(key);
            if (objStr) {
                return JSON.parse(objStr);
            }
            return null;
        },
    },
    
    starredList : {},
}

mc.APIRequest = function($elem, url, storeToCache, offset) {
    this.element = $elem;
    this.url = url;
    this.storeToCache = storeToCache;
    this.retryCount = 0;
    this.maxRetry = 5;
    this.offset = offset;
}

mc.APIRequest.prototype = {
    
    sendRequest : function() {
        var url = this.url;
        var $elem = this.element;
        var storeToCache = this.storeToCache;
        var _this = this;
        
        mc.api.showNetworkIndicator(true);
        
        var _request = $.getJSON(url, function(data) { if (storeToCache) { if (data) { mc.data.cache.storeObject(url, data); } } mc.UI.onData($elem, data, false, _this); });
        _request.error(function() { mc.UI.onData($elem, null, false, _this); });
        _request.complete(function() { mc.api.showNetworkIndicator(false); });
        
        mc.log("sent request to " + url);
        
        
    },
    
    retry : function(delay) {
        if (typeof(delay)=="undefined") { delay = 0; }
        
        var _this = this;
        this.retryCount++;
        if (this.retryCount < this.maxRetry) {
            setTimeout(function() { _this.sendRequest(); }, delay);
            return true;
        }
        return false;
    },
    
    
}

mc.UI = {
    TYPE_PUSH : 'push',
    TYPE_POP : 'pop',
    TYPE_REPLACE : 'replace',
    TYPE_ROOT : 'root',
    TYPE_ROOT_PLUS_1 : 'root_plus_1',
    
    screenshotBase : 'http://s3.amazonaws.com/13thlab_minecloud_screenshot/', //'resources/images/ex/',
	resizeBase : 'http://images.pointcloud.io/?url=',
    currentPage : null,
    currentSearchQuery : "",
    lastPage : null,
    rootPages : [ 'home', 'discover', 'starred', 'search', 'more' ],
    history : [],
    lastTabIndex : -1,
    originalPages : {},
    invalidatedPages : {},
    checksumUsed : false,
    
    startUp : function(position) {
        if (!mc.UI.currentPage) {
            var pageId = this.rootPages[mc.api.currentTabIndex];
            this.openPage(pageId, null, mc.UI.TYPE_ROOT);
        }
    },
    
    openURI : function(uri) {
        mc.api.setSearchbarVisible(false);
        
        var uriObj = this.parseURI(uri);
        if (uriObj && uriObj.type == "world" && uriObj.action == "open") {
            // Look if we should display the video page
            var count = 0;
            var infoDismissed = mc.data.getData("info_dismissed", "false");
            var infoDisplayCount = mc.data.getData("info_display_count");
            
            if (infoDisplayCount) {
                count = parseInt(infoDisplayCount);
            }
            
            if (infoDismissed == "true" || count >= 0) {
                // No need to show interstitual page again
                mc.api.openURI(uri);
                
            }
            else {
                count++;
                mc.data.storeData("info_display_count", ""+count);
                
                var templateValues = { open_uri : uri, view_count : count };
                mc.UI.openPage('place_world_video_page', templateValues, mc.UI.TYPE_PUSH);
                
            }
        }
        else {
            mc.api.openURI(uri);
        }
    },
    
    parseURI : function(uriStr) {
        try {
            var prefix = "minecraftreality://";
            if (uriStr.indexOf(prefix) == 0) {
                uriStr = uriStr.substring(prefix.length);
            }
            
            var parts = uriStr.split("/");
            
            if (parts.length > 1) {
                var uriObj = {};
                uriObj.type = parts[0];
                uriObj.itemId = parts[1];
                uriObj.action = parts[2];
                return uriObj;
            }
        }
        catch (ex) {
            alert(ex);
        }
        return null;
    },
    
    share : function(uri, title, screenshotURL) {
        mc.api.share(uri, title, screenshotURL);
    },
    
    
    onTabSelected : function(index) {
        if (mc.context.position) {
            var _this = this;
            var fnc = function() {
                var pageId = _this.rootPages[index];
                if (_this.lastTabIndex != mc.api.currentTabIndex) {
                    // If there's history here, go to that page
                    var entry = _this.getCurrentEntry();
                    if (entry) {
                        _this.openPage(entry.page, entry.values, mc.UI.TYPE_REPLACE);
                    }
                    else {
                        _this.openPage(pageId, null, mc.UI.TYPE_ROOT);
                    }
                }
                else {
                    // Same tab, always go to root
                    _this.openPage(pageId, null, mc.UI.TYPE_ROOT);
                }
                _this.lastTabIndex = mc.api.currentTabIndex;
            }
            
            setTimeout(fnc, 10);
        }
    },
    
    selectTopLink : function(elem) {
        var $elem = $(elem);
        var $siblings = $elem.parent().find(".toplink");
        $siblings.removeClass("selected");
        
        $elem.addClass("selected");
    },
    
    setDataBoundEnabled : function(databoundElement, enabled) {
        var $databoundElement = $("#"+databoundElement);
        
        if (enabled) {
            $databoundElement.addClass("enabled");
            mc.UI.handleSearch(mc.UI.currentSearchQuery);
        }
        else {
            $databoundElement.removeClass("enabled");
        }
    },
    
    onPullToRefresh : function() {
        if (mc.UI.currentPage) {
            mc.UI.refreshPageData();
            
            // For now
            if (mc.UI.currentPage.attr("id") == "more") {
                location.reload(true);
            }
        }
    },
    
    refreshPageData : function() {
        if (mc.UI.currentPage) {
            var $dataBound = mc.UI.currentPage.find(".databound");
            $dataBound.each(function() { mc.UI.checkData($(this), false, true); });
        }
    },
    
    // History
    getStack : function(tabIndex) {
        if (typeof(tabIndex)=="undefined") { tabIndex = mc.api.currentTabIndex; }
        var stack = mc.UI.history[tabIndex];
        if (!stack) {
            stack = [];
            mc.UI.history[tabIndex] = stack;
        }
        return stack;
    },
    
    pushToStack : function(pageId, templateValues, tabIndex) {
        if (typeof(tabIndex)=="undefined") { tabIndex = mc.api.currentTabIndex; }
        var stack = this.getStack(tabIndex);
        var entry = { page : pageId, values : templateValues };
        stack.push(entry);
        mc.UI.history[tabIndex] = stack;
    },
    
    getCurrentEntry : function() {
        var stack = this.getStack();
        if (stack.length > 0) {
            var entry = stack[stack.length-1];
            return entry;
        }
        return null;
    },
    
    popStack : function(tabIndex) {
        if (typeof(tabIndex)=="undefined") { tabIndex = mc.api.currentTabIndex; }
        var stack = this.getStack(tabIndex);
        if (stack.length > 1) {
            var entry = stack[stack.length-2];
            stack.splice(stack.length-1,1);
            mc.UI.history[tabIndex] = stack;
            return entry;
        }
        return null;
    },
    
    clearStack : function(tabIndex) {
        if (typeof(tabIndex)=="undefined") { tabIndex = mc.api.currentTabIndex; }
        mc.UI.history[tabIndex] = [];
    },
    
    hasBack : function(tabIndex) {
        if (typeof(tabIndex)=="undefined") { tabIndex = mc.api.currentTabIndex; }
        var stack = this.getStack(tabIndex);
        return stack && stack.length > 1; // Root is always there
    },
    
    onBackButton : function() {
        var entry = this.popStack();
        if (entry) {
            mc.UI.openPage(entry.page, entry.values, mc.UI.TYPE_POP);
        }
    },
    
    onRightButton : function(title) {
        var fnc = function() {
            var $page = mc.UI.currentPage;
            var action = $page.data("customright_action");
            var name = $page.data("customright_name");
            if (name == title && action) {
                eval(action);
            }
        }
        setTimeout(fnc, 10);
    },
    
    onKeyboardVisibilityChange : function(visible) {
        var $page = mc.UI.currentPage;
        if ($page) {
            if (visible) {
                $page.addClass("keyboard");
            }
            else {
                $page.removeClass("keyboard");
            }
        }
    },
    
    toggleDebugMode : function() {
        var $log = $("#pathlog");
        if ($log.hasClass("visible")) {
            $log.removeClass("visible");
        }
        else {
            $log.addClass("visible");
        }
    },
    
    debugStack : function() {
        var stack = mc.UI.getStack();
        if (stack) {
            var c = '';
            for (var i=0; i < stack.length; i++) {
                var entry = stack[i];
                var page = entry.page;
                if (i > 0) {
                    c += ' > ';
                }
                c += page;
            }
            
            
            $("#pathlog").html(c);
        }
    },
    
    invalidatePage : function(pageId) {
        this.invalidatedPages[pageId] = true;
    },
    
    beforePageView : function(pageId) {
        var $page = $("#" + pageId);
        
        var $videos = $page.find(".video");
        if ($videos.length > 0) {
            var fnc = function() {
                var src = $(this).attr("data-href");
                var w = window.innerWidth*0.9;
                var h = w*9/16;
                var x = (window.innerWidth-w)/2;
                
                var c = '';
                c += '<embed id="yt" class="youtube-player" style="margin:10px 0px 0px '+x+'px;width:'+w+'px;height:'+h+'px;"';
                c += ' src="'+src+'" type="application/x-shockwave-flash" width="'+w+'" height="'+h+'" allowscriptaccess="always"></embed>';
                
                $(this).html(c);
            };
            
            $videos.each(fnc);
        }
        
        
    },
    
    afterPageLeave : function(pageId) {
        var $page = $("#" + pageId);
        $page.find(".video").html("");
        
        // Images in lists
        var $icons = $page.find(".opened_only");
        var fnc = function() {
            var $elem = $(this);
            var imgURL = $elem.attr("data-href");
            if (imgURL) {
                $elem.css("background-image", "");
            }
        };
        $icons.each(fnc);
    },
    
    prepareDeviceLayout : function() {
        var $body = $(document.body);
        
        if (!$body.data("device_prepared")) {
            var $deviceLayouts = $body.find(".devicelayout");
            var removeList = [];
            var fnc = function() {
                var $elem = $(this);
                var deviceInfoStr = $elem.attr("data-device");
                if (deviceInfoStr) {
                    var deviceInfoData = JSON.parse(deviceInfoStr);
                    
                    if (deviceInfoData) {
                        var onlyFor = deviceInfoData["only-for"];
                        var notFor = deviceInfoData["not-for"];
                       
                        // Only-for
                        if (onlyFor && onlyFor.length > 0 && onlyFor.indexOf(mc.context.deviceInfo.type) == -1) {
                            // Remove
                            removeList.push($elem);
                        }
                        
                        // Not-for
                        if (notFor && notFor.length > 0 && notFor.indexOf(mc.context.deviceInfo.type) != -1) {
                            // Remove
                            removeList.push($elem);
                        }
                    }
                }
            };
            
            $deviceLayouts.each(fnc);
            
            // Remove views not relevant for the device
            for (var i in removeList) {
                var $e = removeList[i];
                $e.remove();
            }
            
            $body.data("device_prepared", true);
        }
    },
    
    
    reportError : function() {
        var subject = "Minecraft Reality Problems";
        var body = "Describe your problem below. Thank you for helping us improve the product.";
        var uri = "mailto://minecraftreality@13thlab.com?subject="+encodeURIComponent(subject)+"&body="+encodeURIComponent(body);
        mc.api.openURI(uri);
    },
    
    openPage : function(pageId, templateValues, type, tabIndex, clearOldStack) {
        if (typeof(tabIndex)=="undefined") { tabIndex = mc.api.currentTabIndex; }
        if (typeof(templateValues)=="undefined") { templateValues = null; }
        if (typeof(type)=="undefined") { type = mc.UI.TYPE_PUSH; }
        
        var $page = $("#"+pageId);
        
        mc.UI.lastPage = mc.UI.currentPage;
        mc.UI.currentPage = $page;
        
        mc.UI.beforePageView(pageId);
        
        if (mc.UI.lastPage) {
            var oldId = mc.UI.lastPage.attr("id");
            var origHtml = mc.UI.originalPages[oldId];
            if (origHtml) {
                //mc.UI.lastPage.html();
                mc.UI.lastPage.removeClass("visible");
                
            }
            mc.UI.afterPageLeave(oldId);
            // alert("hiding "+mc.UI.lastPage.attr("id"));
        }
        
        $(".page").removeClass("visible");
        
        var _this = this;
        
        if (!mc.UI.originalPages[pageId]) {
            mc.UI.originalPages[pageId] = $page.html();
        }
        else {
            // Restore to past glory
            //mc.data.setPageData($page.find(".databound").attr("id"), "data_checksum", 0);
            $page.html(mc.UI.originalPages[pageId]);
        }
        
        // Manage view stack
        if (type == mc.UI.TYPE_PUSH) {
            mc.UI.pushToStack(pageId, templateValues);
        }
        else if (type == mc.UI.TYPE_ROOT) {
            mc.UI.clearStack();
            mc.UI.pushToStack(pageId, templateValues);
        }
        else if (type == mc.UI.TYPE_ROOT_PLUS_1) {
            if (tabIndex != mc.api.currentTabIndex) {
                var oldTabIndex = mc.api.currentTabIndex;
                mc.api.setSelectedTab(tabIndex);
                if (clearOldStack) {
                    mc.UI.clearStack(oldTabIndex);
                }
                mc.UI.clearStack(tabIndex);
            }
            
            mc.UI.pushToStack(pageId, templateValues);
        }
        
        // Reset pull to refresh
        mc.api.setPullToRefreshDone();
        
        mc.UI.debugStack();
        
        // Refresh
        var refreshable = $page.hasClass("refreshable");
        mc.api.setPullToRefreshEnabled(refreshable);
        
        
        // Template
        var templateId = $page.attr("data-template");
        if (templateId || templateValues) {
            var templateHtml = templateId ? $("#"+templateId).html().trim() : $page.html().trim();
            if (templateValues) {
                templateHtml = mc.UI.processTemplate(templateHtml, templateValues);
            }
            
            $page.html(templateHtml);
        }
        
        // Title bar
        mc.UI.updateTitlebar($page);
        
        // Track view
        mc.api.trackView(pageId);
        
        // Data refresh
        var $dataBound = $page.find(".databound");
        var isSearch = pageId == "search";
        var force = false;
        var notifyFromCache = true;
        if (_this.invalidatedPages[pageId]) {
            force = true;
            notifyFromCache = false;
            _this.invalidatedPages[pageId] == false;
        }
        
        if (!isSearch || _this.currentSearchQuery != "") {
            $dataBound.each(function() { mc.UI.checkData($(this), true, force); });
        }
        
        // Images
        var $base64Images = $page.find(".base64");
        $base64Images.each(function() { mc.UI.loadBase64Image($(this)); });
        
        // Prepare clickables
        _this.updateObservers($page);
        
        window.scrollTo(0,0);
        
        // Make visible
        $page.addClass("visible");
        
    },
    
    handleSearch : function(query) {
        // Data refresh
        var $page = mc.UI.currentPage;
        if ($page.attr("id")=="search") {
            //if (query == "") { query = "__empty_search_query__"; }
            mc.UI.currentSearchQuery = query;
            var $dataBound = $page.find(".databound");
            $dataBound.each(function() { mc.UI.checkData($(this), false, true); });
        }
    },
    
    loadBase64Image : function($elem) {
        var filePath = $elem.attr("data-href");
        if (filePath) {
            mc.api.requestBase64Image(filePath);
        }
    },
    
    onBase64Image : function(path, data) {
        var $page = mc.UI.currentPage;
        var $images = $(".base64");
        var fnc = function() {
            var _path = $(this).attr("data-href");
            if (path == $(this).attr("data-href")) {
                $(this).css("background-image", "url(data:image/png;base64,"+data+")");
            }
        };
        
        $images.each(fnc);
    },
    
    updateObservers : function($parent) {
        var $clickables = $parent.find(".clickable");
        var fnc = function($elem, evt) {
            var otc = $elem.attr("data-onclick");
            var _this = $elem.get(0);
            eval(otc);
        }
        mc.UI.prepareButtons($clickables, fnc, true);

    },
    
    updateTitlebar : function($page, type) {
        var hasBack = mc.UI.hasBack();
        
        var title = $page.attr("title");
        var rightTitle = "";
        var titleStr = "";
        
        var titleParts = title.split(";");
        titleStr = titleParts[0];
        if (titleParts.length == 2) {
            rightTitle = titleParts[1];
        }
        
        var $customRight = $page.find("._topcustombutton");

        if ($customRight.length > 0) {
            rightTitle = $customRight.html();
            var onclick = $customRight.attr("data-onclick");
            $page.data("customright_action", onclick);
        }
        else {
            $page.data("customright_action", null);
        }
        $page.data("customright_name", rightTitle);
        
        if (rightTitle && rightTitle.indexOf("::search")==0) {
            var parts = rightTitle.split("|");
            var hint = null;
            if (parts.length == 2) {
                hint = parts[1];
            }
            
            mc.api.setSearchbarVisible(true, true, hint);
            rightTitle = "";
        }
        else {
            if (mc.api.searchVisible) {
                mc.api.setSearchbarVisible(false);
                mc.api.clearSearchbar();
            }
        }
        
        mc.api.updateTitlebar(titleStr, hasBack, false, rightTitle);
    },
    
    getPagingData : function($databound) {
        // Default values
        var offset = 0;
        var pageSize = 20;
        var currentPage = 0;
        
        var isPaginated = $databound.hasClass("paginated");
        
        if (isPaginated) {
            var currentPageStr = mc.data.getPageData($databound.attr("id"), "page_number");
            if (currentPageStr) {
                currentPage = parseInt(currentPageStr)
            }
            else {
                currentPage = 0;
            }
            
            var maxCountStr = $databound.find(".data_container").attr("data-maxcount");
            if (maxCountStr) {
                pageSize = parseInt(maxCountStr);
            }
            
            if (pageSize && currentPage) {
                offset = currentPage * pageSize;
            }
            
        }
        
        return {
            offset : offset,
            pageSize : pageSize,
            currentPage : currentPage,
            
        };
    },
    
    checkData : function($elem, notifyFromCache, force, cachedOnly) {
        if (!$elem.hasClass("enabled")) {
            return;
        }
        
        var $section = $elem.parents("section");
        $section.find(".loader").addClass("visible");
        
        var now = new Date().getTime();
        var src = $elem.attr("data-href");
        
        var refreshTime = $elem.attr("data-refresh") ? $elem.attr("data-refresh") : 300;
        var lastUpdate = $elem.data("last-update");
        if (force || (!lastUpdate || (now - lastUpdate > refreshTime*1000))) {
            var pagingData = mc.UI.getPagingData($elem);
            
            
            // Continue
            var templateValues = {
                lat : mc.context.position ? mc.context.position.lat : 0,
                long : mc.context.position ? mc.context.position.lon : 0,
                device_id : mc.context.deviceInfo.id,
                device_type : encodeURIComponent(mc.context.deviceInfo.type),
                query : encodeURIComponent(mc.UI.currentSearchQuery),
                max_count : pagingData.pageSize,
                offset : pagingData.offset,
            };
            for (var k in templateValues) {
                var v = templateValues[k];
                src = src.replaceAll("{"+k+"}", v);
                
            }
            
            var $indicator = $elem.parent().find(".indicator");
            $indicator.addClass("visible");
            
            mc.data.request($elem, src, notifyFromCache, true, cachedOnly);
        }
    },
    
    getTimeAgo : function(date) {
        var diff = new Date().getTime() - date.getTime();
        var seconds = diff/1000;
        var minutes = seconds/60;
        var hours = minutes/60;
        var days = Math.round(hours/24);
        var weeks = Math.round(hours/(24*7));
        var months = Math.round(days/30.5);
        
        if (months > 1) {
            return months + " months ago";
        }
        if (weeks > 1) {
            return weeks + " weeks ago";
        }
        if (days > 1) {
            return days + " days ago";
        }
        if (hours > 1) {
            return Math.round(hours) + " hours ago";
        }
        if (minutes >= 1) {
            return Math.round(minutes) + " minutes ago";
        }
        if (seconds > 30) {
            return Math.round(seconds) + " seconds ago";
        }
        else {
            return "Just now";
        }
    },
    
    getDataChecksum : function(data) {
        var cs = 0;
        if (data) {
            var results = data["result"];
            return results.length + JSON.stringify(data).length;
        }
        return 0;
    },
    
    onData : function($elem, data, isCached, request) {
        if (!isCached) {
            mc.api.setPullToRefreshDone();
        }
        
        if (!data) {
            if (request) {
                var didRetry = request.retry(3000);
                
                mc.log("did retry");
                
                if (!didRetry) {
                    if (confirm("Could not refresh data due to a network error.\n\n Click OK to retry, or Cancel to ignore.")) {
                        mc.UI.refreshPageData();
                    }
                }
            }
            
            return;
        }
        
        if (request && request.url) {
            var currentURL = mc.data.getPageData($elem.attr("id"), "current_url");
            var requestURL = request.url;
            if (currentURL && currentURL != requestURL) {
                // Old data
                //alert('old data, returning\n\n'+ requestURL + ", " + currentURL);
                return;
            }
        }
        
        
        $elem.data("last-update", new Date().getTime());
        
        var databoundTemplateValues = null;
        if ($elem.attr("data-values")) {
            databoundTemplateValues = JSON.parse($elem.attr("data-values"));
        }
        
        var $indicator = $elem.parent().find(".indicator");
        $indicator.removeClass("visible");
        
        if (!data) {
            alert('There was an error. Please try again.');
        }
        
        if (!isCached) {
            
            var $section = $elem.parents("section");
            $section.find(".loader").removeClass("visible");
        }
        
        if (data) {
            var $dataParent = $elem.find(".data_container");
            
            var dataTypeStr = $elem.attr("data-itemtype");
            var dataTypeParts = dataTypeStr ? dataTypeStr.split(":") : [];
            var dataType = dataTypeParts.length > 0 ? dataTypeParts[0] : dataTypeStr;
            var dataTypeExtra = dataTypeParts.length > 1 ? dataTypeParts[1] : null;
            
            var templateId = $dataParent.attr("data-template");
            var templateHtml = $("#"+templateId).html();
            var list = data["result"];
            
            // Prev, next, pageCounter
            // Next/prev
            var $prev = $elem.parents("section").find(".prev");
            var $next = $elem.parents("section").find(".next");
            var $counter = $elem.parents("section").find(".pagecounter");
            
            var pagingData = mc.UI.getPagingData($elem);
            if (pagingData.currentPage > 0) {
                $prev.addClass("visible");
            }
            else {
                $prev.removeClass("visible");
            }
            
            mc.api.log("Data: " + JSON.stringify(data));
            
            if (pagingData.pageSize <= list.length) {
                $next.addClass("visible");
            }
            else {
                $next.removeClass("visible");
            }
            
            if (pagingData.currentPage) {
                $counter.html(" (" + (pagingData.currentPage+1) + ")");
            }
            else {
                $counter.html(" ");
            }
            
            
            var options = $elem.data("options");
            if (!options) {
                var optionStr = $elem.attr("data-options");
                if (optionStr) {
                    options = JSON.parse(optionStr);
                }
                else {
                    options = {};
                }
                $elem.data("options", options);
            }
            
            var $noItems = $elem.find(".nodata");
            if (list && list.length > 0) {
                $noItems.removeClass("visible")
            }
            else {
                $noItems.addClass("visible");
            }
            
            var _continue = true;
            var newHtml = null;
            var errorCode = data.error_code;
            if (errorCode != 0) {
                if (!isCached) {
                    newHtml = "";
                }
                
                if (options && options.clearOnError) {
                    _continue = true;
                }
            }
            
            // Checksum
            var checksum = mc.UI.getDataChecksum(data);
            var oldChecksum = mc.data.getPageData($elem.attr("id"), "data_checksum");
            if (!isCached && oldChecksum && oldChecksum != 0 && oldChecksum == checksum) {
                //_continue = false;
                if (mc.UI.checksumUsed) {
                    _continue = false;
                }
                else {
                    mc.UI.checksumUsed = true;
                }
            }
            else {
                mc.data.setPageData($elem.attr("id"), "data_checksum", checksum);
            }
            
            
            
            if (_continue) {
                var existingHtml = $dataParent.html();
                
                if (dataType == "worldinfo") {
                    var item = list[0];
                    var screenshots = [];
                    for (var i in item.screenshots) {
                        screenshots.push( mc.UI.getScreenshotURL(item.screenshots[i], 280) );
                    }
                    
                    var date = new Date(item.created_on);
                    var timeAgo = mc.UI.getTimeAgo(date);
                    
                    var templateValues = {
                        world_id : item.id,
                        world_name : item.title,
                        world_description : item.desc,
                        world_upload_date : timeAgo,
                        world_screenshots : screenshots,
                    };
                    
                    newHtml = this.processTemplate(templateHtml, templateValues);
                    
                }
                else if (dataType == "imagelist") {
                    var maxCount = $dataParent.attr("data-maxcount") ? $dataParent.attr("data-maxcount") : 20;
                    var count = list.length > maxCount ? maxCount : list.length;
                    
                    newHtml = "";
                    
                    var world_id = databoundTemplateValues && databoundTemplateValues.world_id ? databoundTemplateValues.world_id : '';
                    
                    for (var i=0; i < count; i++) {
                        var item = list[i];
                        var templateValues = {
                            image_url : item.url,
                            encoded_image_url : encodeURIComponent(item.url),
                            image_filename : item.filename,
                            world_id : world_id,
                        }
                        newHtml += this.processTemplate(templateHtml, templateValues);
                    }
                }
                else if (dataType == "placed_worldinfo") {
                    var item = list[0];
                    
                    var screenshots = [];
                    for (var i in item.screenshots) {
                        screenshots.push( mc.UI.getScreenshotURL(item.screenshots[i], 200) );
                    }
                    
                    var refImages = [];
                    if (item.map_type == 1) {
                        // Image
                        var refImageSrc = decodeURIComponent(item.map_id);
                        refImages.push(refImageSrc);
                    }
                    
                    var distance = item.lat && item.long ? mc.UI.getDistance(item.lat, item.long, true, true) : "-";
                    
                    var templateValues = {
                        placed_world_id : item.id,
                        placed_world_world_id : item.world_id,
                        placed_world_name : item.title,
                        placed_world_description : item.desc,
                        placed_world_upload_date : item.created_on,
                        placed_world_screenshots : screenshots,
                        placed_world_ref_image : refImages,
                        placed_world_starred_state : (item.device_starred ? "on" : ""),
                        placed_world_distance : distance,
                        placed_world_starcount : item.starred + " stars",
                        placed_world_viewcount : item.opened_place + " views",
                    };
                    
                    newHtml = this.processTemplate(templateHtml, templateValues);
                }
                else if (dataType == "worldlist") {
                    var maxCount = $dataParent.attr("data-maxcount") ? $dataParent.attr("data-maxcount") : 20;
                    var count = list.length > maxCount ? maxCount : list.length;
                    var screenshotSize = dataTypeExtra && dataTypeExtra == "large" ? 2*280 : 100;
                    
                    newHtml = "";
                    
                    for (var i=0; i < count; i++) {
                        var item = list[i];
                        var screenshotURL = "";
                        if (item.screenshots) {
                            screenshotURL = mc.UI.getScreenshotURL(item.screenshots[0], screenshotSize);
                            
                            mc.log("URL: " + screenshotURL);
                        }
                        else {
                            screenshotURL = "resources/images/noworldphoto.jpg";
                        }
                        
                        var templateValues = {
                            world_id : item.id,
                            world_name : item.title,
                            world_icon : screenshotURL,
                            world_description : item.desc,
                            
                        };
                        newHtml += this.processTemplate(templateHtml, templateValues);
                    }
                }
                else if (dataType == "placed_worldlist" || dataType == "starred_placed_worldlist") {
                    var maxCount = $dataParent.attr("data-maxcount") ? $dataParent.attr("data-maxcount") : 20;
                    var count = list.length > maxCount ? maxCount : list.length;
                    var screenshotSize = dataTypeExtra && dataTypeExtra == "large" ? 2*280 : 100;
                    
                    newHtml = ""; //count > 0 ? "" : ((dataType == "placed_wordlist") ? "" : null);
                    
                    if (count > 0 && dataType == "starred_placed_worldlist") {
                        mc.data.starredList = {};
                    }
                    
                    for (var i=0; i < count; i++) {
                        var item = list[i];
                        var screenshotURL = mc.UI.getScreenshotURL(item.screenshots[0], screenshotSize);
                        var distance = mc.UI.getDistance(item.lat, item.long, true, true);
                        
                        var templateValues = {
                            placed_world_id : item.id,
                            placed_world_name : item.title,
                            placed_world_icon : screenshotURL,
                            placed_world_description : item.desc,
                            placed_world_distance : distance,
                        };
                        
                        // Store to starred cache
                        if (dataType == "starred_placed_worldlist") {
                            mc.data.starredList[item.id] = item;
                        }
                        
                        newHtml += this.processTemplate(templateHtml, templateValues);
                        
                        
                    }
                }
                
            }
            
            if (newHtml && existingHtml) {
                //alert("A: " + newHtml.length + " vs " + existingHtml.length);
            }
            
            if (newHtml != null && newHtml != existingHtml.trim()) {
                //alert("B: " + newHtml);
                if (existingHtml) {
                    
                    //alert(newHtml.length + " vs " + existingHtml.length);
                    //alert(newHtml + "("+newHtml.length+") vs " + existingHtml);
                    //alert($dataParent.find("li").length + " vs " + $(newHtml).find("li").length);
                    //alert($dataParent.html());
                }
                
                $dataParent.html(newHtml); 
                
                //alert("setting new html: " + $dataParent.parents(".section").html());
                //alert($dataParent.attr("data-template"));
                
                
            }
            
            try {
                // Update title bar
                mc.UI.updateTitlebar(mc.UI.currentPage);
                
                // Prepare clickables
                this.updateObservers($dataParent);
            }
            catch (ex) {
                alert("errorapa:"+ex);
            }

        }
    },
    
    processTemplate : function(html, templateValues) {
        // Handle loops
        var loops = html.split("{endforeach}");
        
        if (loops.length > 1) {
            var _html = "";
            for (var l in loops) {
                var partHtml = loops[l] + (l < loops.length-1 ? "{endforeach}" : "");
                
                var start = partHtml.indexOf("{foreach");
                while (start != -1) {
                    var end = partHtml.indexOf("{endforeach}") + "{endforeach}".length;
                    var loop = partHtml.substring(start, end);
                    
                    var loopDataStr = loop.substring(loop.indexOf(":")+1, loop.indexOf("}"));
                    var loopData = loopDataStr.split(":");
                    var loopVariable = loopData[0];
                    var loopIndexName = loopData[1];
                    var loopMaxCount = loopData[2] ? loopData[2] : 100;
                    var loopHtml = loop.substring(loop.indexOf("}")+1, loop.indexOf("{endforeach}"));
                    
                    var c = '';
                    var data = templateValues[loopVariable];
                    var count = data ? data.length : 0;
                    if (count > loopMaxCount) {
                        count = loopMaxCount;
                    }
                    
                    for (var i=0; i < count; i++) {
                        var trname = "{" + loopVariable + "[" + loopIndexName + "]}";
                        c += loopHtml.replace(trname, data[i]);
                    }
                    partHtml = partHtml.substring(0,start) + c + partHtml.substring(end);
                    start = partHtml.indexOf("{foreach}");
                }
                
                _html += partHtml;
                
            }
            html = _html;
        }
        
        for (var k in templateValues) {
            var v = templateValues[k];
            html = html.replaceAll("{"+k+"}", v);
        }
        return html;
    },
    
    handlePrev : function(elem) {
        // Elem
        var $elem = $(elem);
        var $section = $elem.parents("section");
        var $databound = $section.find(".databound");
        var pageId = $databound.attr("id");
        
        var currentPage = mc.data.getPageData(pageId, "page_number"); //setPageData
        if (!currentPage) {
            currentPage = 0;
        }
        
        currentPage--;
        
        if (currentPage < 0) {
            currentPage = 0;
            
        }
        
        mc.data.setPageData(pageId, "page_number", currentPage);
        
        $databound.find("li").css("background-image", "none");
        
        $databound.each(function() { mc.UI.checkData($(this), true, true, false); });
        
       
    },
    
    
    handleNext : function(elem) {
        // Elem
        var $elem = $(elem);
        var $section = $elem.parents("section");
        var $databound = $section.find(".databound");
        var pageId = $databound.attr("id");
        
        var currentPage = mc.data.getPageData(pageId, "page_number"); //setPageData
        if (!currentPage) {
            currentPage = 0;
        }
        
        currentPage++;
        
        mc.data.setPageData(pageId, "page_number", currentPage);
    
        $databound.find("li").css("background-image", "none");
        
        $databound.each(function() { mc.UI.checkData($(this), true, true, false); });
        
    },
    
    
    prepareButtons : function($elem, fnc, cancelEvent) {
        if (typeof(cancelEvent)=="undefined") { cancelEvent = true; }
        var	maxMove = 4;
        
        $elem.unbind();
        
        $elem.bind("touchstart", function(e) {
                   var disabled = $(this).hasClass("disabled");
                   if (!disabled) {
                   $(this).data("touch", "true");
                   var $elem = $(this);
                   setTimeout(function() { if ($elem.data("moved_out")==false) { $elem.addClass("active"); } }, 50);
                   var touch = e.originalEvent.touches[0] || e.originalEvent.changedTouches[0];
                   $(this).data("touchdown", {x: touch.pageX, y: touch.pageY});
                   $(this).data("moved_out", false);
                   }
                   });
        
        
        $elem.bind("touchmove", function(e) {
                   
                   if ($(this).data("moved_out") == false) {
                   var touch = e.originalEvent.touches[0] || e.originalEvent.changedTouches[0];
                   var touchDown = $(this).data('touchdown');
                   var maxMoveSq = Math.pow((touchDown.x - touch.pageX),2) + Math.pow((touchDown.y - touch.pageY), 2);
                   
                   if (maxMoveSq > Math.pow(maxMove,2)) {
                   $(this).removeClass("active");
                   $(this).data("moved_out", true);
                   }
                   }
                   });
        
        $elem.bind("touchend touchcancel", function(e) {
                   $(this).removeClass("active");
                   var disabled = $(this).hasClass("disabled");
                   if (!disabled) {
                   if ($(this).data("moved_out") == false) {
                   var touch = e.originalEvent.touches[0] || e.originalEvent.changedTouches[0];
                   var touchDown = $(this).data('touchdown');
                   var maxMoveSq = Math.pow((touchDown.x - touch.pageX),2) + Math.pow((touchDown.y - touch.pageY), 2);
                   
                   if (maxMoveSq < Math.pow(maxMove,2)) {
                   fnc($(this), e);
                   }
                   }
                   
                   }
                   
                   
                   if (cancelEvent) {
                   e.preventDefault();
                   e.stopPropagation();
                   }
                   });
        
        
    },
    
    onWorldViewDone : function() {
        
    },
    
    // Specifics
    
    dismissIntroVideo : function(uri) {
        mc.data.storeData("info_dismissed", "true");
        if (uri) {
            mc.UI.openURI(uri);
        }
    },
    
    onWorldPlaced : function(data) {
        var screenshots = [];
        for (var i in data.screenshots) {
            var filename = data.screenshots[i];
            if (filename.indexOf("with_content") != -1) {
                screenshots.push(filename);
            }
        }
        
        var templateValues = {
            world_id : data.world_id,
            placed_world_screenshots : screenshots,
        }
        mc.UI.openPage("save_placed_world", templateValues, mc.UI.TYPE_PUSH);
    },
    
    onWorldSaved : function(data) {
        
        var id = data.result[0];
        if (id) {
            mc.UI.openPage("placedworldinfo", { placedworld_id : id }, mc.UI.TYPE_REPLACE);
        }
        else {
            alert("There was an error while saving world");
        }
    },
    
    removeScreenshot : function(elem) {
        var $elem = $(elem);
        var $parent = $elem.parent(".screenshot");
        
        var $list = $parent.parents("ul");
        $parent.data("gone", true).fadeOut(function() { $(this).remove(); });
       
        var $items = $list.find(".screenshot");
        var count = 0;
        $items.each(function() { if (!$(this).data("gone")) { count++; } });
        if (count == 1) {
            $list.find(".screenshot .remove").fadeOut(function() { $(this).remove(); });
        }
    },
    
    saveWorld : function(id) {
        var $page = $("#save_placed_world");
        var $inputRows = $page.find(".inputrow");
        
        var $name = $page.find("#i_placed_world_name");
        var $desc = $page.find("#i_placed_world_description");
        
        var name = $name.val();
        var desc = $desc.val();
        var screenshots = [];
        
        var submit = true;
        
        if (!name) {
            mc.UI.shakeElement($name);
            submit = false;
        }
        
        if (!desc) {
            mc.UI.shakeElement($desc);
            submit = false;
        }
        
        if (submit) {
            // Hide the keyboard
            $name.blur();
            $desc.blur();
            
            $page.find(".screenshot").each(function() {
                var path = $(this).attr("data-href");
                var pathNoContentVersion = path.replace("_with_content", "");
                screenshots.push(path);
                screenshots.push(pathNoContentVersion);
            });
            mc.api.saveWorld(id, name, desc, screenshots);
        }
    },
    
    shakeElement : function($elem) {
		$elem.addClass("shake");
		setTimeout(function() { $elem.removeClass("shake"); }, 1000);
	},
    
    toggleStar : function(elem, placedWorldId) {
        //alert(placedWorldId);
        var $elem = $(elem);
        if ($elem.hasClass("on")) {
            $elem.removeClass("on");
            // Star world
            mc.data.setPlacedWorldStarred(placedWorldId, false);
        }
        else {
            $elem.addClass("on");
            mc.data.setPlacedWorldStarred(placedWorldId, true);
        }
        this.invalidatePage("starred");
    },
    
    
    
    // Utils
    getScreenshotURL : function(screenshotId, width) {
		var imageURL = mc.UI.screenshotBase+screenshotId;
	    
        var parts = screenshotId.split("_");
        var worldId = parts[1];
        var img = parts[2];
        
        imageURL = "/Data/" + worldId + "/screenshots/" + img + ".png";
        
        mc.api.log("Image url: " + screenshotId + " => " + imageURL);
        
        /*if (width) {
            return mc.UI.resizeBase + encodeURIComponent(imageURL) + "&w=" + width;
        }
         */
		return imageURL;
	},
    
    getTime : function(isoTime) {
        
    },
    
    getDistance : function(lat1, lon1, approx, asText) {
		if (mc.context.position.lat == 0.0) {
			return;
		}
		
		var lat2 = mc.context.position.lat;
		var lon2 = mc.context.position.lon;
		
		
		var R = 63710; // km
		var dist = 0;
		
		if (approx) {
			// Seems to be off by a factor of 10
			var x = (lon2-lon1) * Math.cos((lat1+lat2)/2);
			var y = (lat2-lat1);
			dist = Math.sqrt(x*x + y*y) * R;
		}
		else {
			var dLat = (lat2-lat1).toRad();
			var dLon = (lon2-lon1).toRad();
			var lat1 = lat1.toRad();
			var lat2 = lat2.toRad();
			
			var a = Math.sin(dLat/2) * Math.sin(dLat/2) +
			Math.sin(dLon/2) * Math.sin(dLon/2) * Math.cos(lat1) * Math.cos(lat2);
			var c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a));
			dist = R * c;
		}
		
		if (asText) {
			if (dist < 5) {
				return "0-5 m";
			}
			else if (dist < 10) {
				return "5 m";
			}
			else if (dist < 15) {
				return "10 m";
			}
			else if (dist < 20) {
				return "15 m";
			}
			else if (dist < 25) {
				return "20 m";
			}
			else if (dist < 50) {
				return "30 m";
			}
			else if (dist < 75) {
				return "50 m";
			}
			else if (dist < 100) {
				return "75 m";
			}
			else if (dist < 150) {
				return "100 m";
			}
			
			return (0.1*Math.round(dist/100)).toFixed(1) + " km";
		}
		return dist;
	},


}

function testOnWorldPlaced() {
    var cmd = "mc.api.onAppEvent('world_placed', { world_id : '506ec1fd8f20e7599c000000', screenshots : ['/var/mobile/Applications/BEF07792-686C-479D-BC30-EBCF56552EA2/Documents/temp/screenshot0.jpg','/var/mobile/Applications/BEF07792-686C-479D-BC30-EBCF56552EA2/Documents/temp/screenshot1.jpg'] } );";
    eval(cmd);
}


